@@ -1,23 +1,6 @@
 /*
- * JBoss, Home of Professional Open Source.
- * Copyright 2008, Red Hat Middleware LLC, and individual contributors
- * as indicated by the @author tags. See the copyright.txt file in the
- * distribution for a full listing of individual contributors.
- *
- * This is free software; you can redistribute it and/or modify it
- * under the terms of the GNU Lesser General Public License as
- * published by the Free Software Foundation; either version 2.1 of
- * the License, or (at your option) any later version.
- *
- * This software is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this software; if not, write to the Free
- * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
- * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
+ * Copyright The WildFly Authors
+ * SPDX-License-Identifier: Apache-2.0
  */
 package org.jboss.as.ejb3.iiop;
 
@@ -457,11 +440,11 @@ public class EjbCorbaServant extends Servant implements InvokeHandler, LocalIIOP
                     return context.getAuthorizedIdentity();
                 } else {
                     context.fail();
-                    throw new SecurityException("Authorization failed");
+                    throw EjbLogger.ROOT_LOGGER.authenticationFailed();
                 }
             } else {
                 context.fail();
-                throw new SecurityException("Authentication failed");
+                throw EjbLogger.ROOT_LOGGER.authenticationFailed();
             }
         } catch (IllegalArgumentException | IllegalStateException | RealmUnavailableException e) {
             context.fail();
