@@ -1,38 +1,26 @@
 /*
- * JBoss, Home of Professional Open Source.
- * Copyright 2011, Red Hat, Inc., and individual contributors
- * as indicated by the @author tags. See the copyright.txt file in the
- * distribution for a full listing of individual contributors.
- *
- * This is free software; you can redistribute it and/or modify it
- * under the terms of the GNU Lesser General Public License as
- * published by the Free Software Foundation; either version 2.1 of
- * the License, or (at your option) any later version.
- *
- * This software is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this software; if not, write to the Free
- * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
- * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
+ * Copyright The WildFly Authors
+ * SPDX-License-Identifier: Apache-2.0
  */
 package org.wildfly.extension.messaging.activemq;
 
-import static org.wildfly.extension.messaging.activemq.logging.MessagingLogger.ROOT_LOGGER;
+import static org.wildfly.extension.messaging.activemq._private.MessagingLogger.ROOT_LOGGER;
 
 import static java.util.concurrent.TimeUnit.SECONDS;
 import static org.jboss.as.controller.descriptions.ModelDescriptionConstants.PATH;
 import static org.jboss.as.controller.security.CredentialReference.handleCredentialReferenceUpdate;
 import static org.jboss.as.controller.security.CredentialReference.rollbackCredentialStoreUpdate;
+import static org.wildfly.extension.messaging.activemq.AddressSettingAdd.createDefaulAddressSettings;
 import static org.wildfly.extension.messaging.activemq.Capabilities.ACTIVEMQ_SERVER_CAPABILITY;
 import static org.wildfly.extension.messaging.activemq.Capabilities.DATA_SOURCE_CAPABILITY;
 import static org.wildfly.extension.messaging.activemq.Capabilities.ELYTRON_DOMAIN_CAPABILITY;
+import static org.wildfly.extension.messaging.activemq.Capabilities.ELYTRON_SSL_CONTEXT_CAPABILITY_NAME;
 import static org.wildfly.extension.messaging.activemq.Capabilities.HTTP_UPGRADE_REGISTRY_CAPABILITY_NAME;
 import static org.wildfly.extension.messaging.activemq.Capabilities.JMX_CAPABILITY;
+import static org.wildfly.extension.messaging.activemq.Capabilities.OUTBOUND_SOCKET_BINDING_CAPABILITY;
+import static org.wildfly.extension.messaging.activemq.Capabilities.OUTBOUND_SOCKET_BINDING_CAPABILITY_NAME;
 import static org.wildfly.extension.messaging.activemq.Capabilities.PATH_MANAGER_CAPABILITY;
+import static org.wildfly.extension.messaging.activemq.Capabilities.SOCKET_BINDING_CAPABILITY_NAME;
 import static org.wildfly.extension.messaging.activemq.CommonAttributes.ADDRESS_SETTING;
 import static org.wildfly.extension.messaging.activemq.CommonAttributes.BINDINGS_DIRECTORY;
 import static org.wildfly.extension.messaging.activemq.CommonAttributes.HTTP_ACCEPTOR;
@@ -49,6 +37,7 @@ import static org.wildfly.extension.messaging.activemq.CommonAttributes.PAGING_D
 import static org.wildfly.extension.messaging.activemq.CommonAttributes.SECURITY_SETTING;
 import static org.wildfly.extension.messaging.activemq.PathDefinition.PATHS;
 import static org.wildfly.extension.messaging.activemq.PathDefinition.RELATIVE_TO;
+import static org.wildfly.extension.messaging.activemq.ServerDefinition.ADDRESS_QUEUE_SCAN_PERIOD;
 import static org.wildfly.extension.messaging.activemq.ServerDefinition.ASYNC_CONNECTION_EXECUTION_ENABLED;
 import static org.wildfly.extension.messaging.activemq.ServerDefinition.CLUSTER_PASSWORD;
 import static org.wildfly.extension.messaging.activemq.ServerDefinition.CLUSTER_USER;
@@ -107,6 +96,7 @@ import static org.wildfly.extension.messaging.activemq.ServerDefinition.PERSISTE
 import static org.wildfly.extension.messaging.activemq.ServerDefinition.PERSIST_DELIVERY_COUNT_BEFORE_DELIVERY;
 import static org.wildfly.extension.messaging.activemq.ServerDefinition.PERSIST_ID_CACHE;
 import static org.wildfly.extension.messaging.activemq.ServerDefinition.SCHEDULED_THREAD_POOL_MAX_SIZE;
+import static org.wildfly.extension.messaging.activemq.ServerDefinition.SECURITY_DOMAIN;
 import static org.wildfly.extension.messaging.activemq.ServerDefinition.SECURITY_ENABLED;
 import static org.wildfly.extension.messaging.activemq.ServerDefinition.SECURITY_INVALIDATION_INTERVAL;
 import static org.wildfly.extension.messaging.activemq.ServerDefinition.SERVER_DUMP_INTERVAL;
@@ -131,6 +121,7 @@ import javax.management.MBeanServer;
 import javax.sql.DataSource;
 
 import io.undertow.server.handlers.ChannelUpgradeHandler;
+import javax.net.ssl.SSLContext;
 import org.apache.activemq.artemis.api.core.BroadcastGroupConfiguration;
 import org.apache.activemq.artemis.api.core.DiscoveryGroupConfiguration;
 import org.apache.activemq.artemis.api.core.Interceptor;
@@ -141,12 +132,13 @@ import org.apache.activemq.artemis.core.config.CoreQueueConfiguration;
 import org.apache.activemq.artemis.core.config.impl.ConfigurationImpl;
 import org.apache.activemq.artemis.core.config.storage.DatabaseStorageConfiguration;
 import org.apache.activemq.artemis.core.security.Role;
-import org.apache.activemq.artemis.core.server.ActiveMQServer;
 import org.apache.activemq.artemis.core.server.JournalType;
 import org.apache.activemq.artemis.core.settings.impl.AddressSettings;
 import org.apache.activemq.artemis.utils.critical.CriticalAnalyzerPolicy;
 import org.jboss.as.controller.AbstractAddStepHandler;
 import org.jboss.as.controller.AttributeDefinition;
+import org.jboss.as.controller.CapabilityServiceBuilder;
+import org.jboss.as.controller.CapabilityServiceTarget;
 import org.jboss.as.controller.ObjectTypeAttributeDefinition;
 import org.jboss.as.controller.OperationContext;
 import org.jboss.as.controller.OperationFailedException;
@@ -167,13 +159,12 @@ import org.jboss.msc.service.ServiceBuilder;
 import org.jboss.msc.service.ServiceController;
 import org.jboss.msc.service.ServiceController.Mode;
 import org.jboss.msc.service.ServiceName;
-import org.jboss.msc.service.ServiceTarget;
 import org.jboss.msc.value.InjectedValue;
 import org.wildfly.common.function.ExceptionSupplier;
 import org.wildfly.extension.messaging.activemq.broadcast.BroadcastCommandDispatcherFactory;
 import org.wildfly.extension.messaging.activemq.ha.HAPolicyConfigurationBuilder;
 import org.wildfly.extension.messaging.activemq.jms.JMSService;
-import org.wildfly.extension.messaging.activemq.logging.MessagingLogger;
+import org.wildfly.extension.messaging.activemq._private.MessagingLogger;
 import org.wildfly.security.auth.server.SecurityDomain;
 import org.wildfly.security.credential.source.CredentialSource;
 
@@ -193,9 +184,6 @@ class ServerAdd extends AbstractAddStepHandler {
     private static final String ARTEMIS_BROKER_CONFIG_JBDC_LOCK_EXPIRATION_MILLIS = "brokerconfig.storeConfiguration.jdbcLockExpirationMillis";
     private static final String ARTEMIS_BROKER_CONFIG_JDBC_LOCK_ACQUISITION_TIMEOUT_MILLIS = "brokerconfig.storeConfiguration.jdbcLockAcquisitionTimeoutMillis";
 
-    private static final ServiceName SECURITY_BOOTSTRAP_SERVICE = ServiceName.JBOSS.append("security", "bootstrap");
-    private static final ServiceName SECURITY_DOMAIN_SERVICE = ServiceName.JBOSS.append("security", "security-domain");
-
     final BiConsumer<OperationContext, String> broadcastCommandDispatcherFactoryInstaller;
 
     ServerAdd(BiConsumer<OperationContext, String> broadcastCommandDispatcherFactoryInstaller) {
@@ -213,9 +201,6 @@ class ServerAdd extends AbstractAddStepHandler {
     @Override
     protected void populateModel(OperationContext context, ModelNode operation, Resource resource) throws OperationFailedException {
         super.populateModel(context, operation, resource);
-        if(SECURITY_ENABLED.resolveModelAttribute(context, operation).asBoolean()) {
-            context.addStep(SecurityDomainCheckHandler.INSTANCE, OperationContext.Stage.MODEL);
-        }
         handleCredentialReferenceUpdate(context, resource.getModel().get(CREDENTIAL_REFERENCE.getName()), CREDENTIAL_REFERENCE.getName());
 
         // add an operation to create all the messaging paths resources that have not been already been created
@@ -293,7 +278,7 @@ class ServerAdd extends AbstractAddStepHandler {
 
         @Override
         public void execute(OperationContext context, ModelNode operation) throws OperationFailedException {
-            final ServiceTarget serviceTarget = context.getServiceTarget();
+            final CapabilityServiceTarget capabilityServiceTarget= context.getCapabilityServiceTarget();
 
             final String serverName = context.getCurrentAddressValue();
 
@@ -313,14 +298,13 @@ class ServerAdd extends AbstractAddStepHandler {
 
             // Add the ActiveMQ Service
             ServiceName activeMQServiceName = MessagingServices.getActiveMQServiceName(serverName);
-            final ServiceBuilder serviceBuilder = serviceTarget.addService(activeMQServiceName);
-            Supplier pathManager = serviceBuilder.requires(context.getCapabilityServiceName(PATH_MANAGER_CAPABILITY, PathManager.class));
+            final CapabilityServiceBuilder serviceBuilder = capabilityServiceTarget.addCapability(ACTIVEMQ_SERVER_CAPABILITY);
+            Supplier<PathManager> pathManager = serviceBuilder.requiresCapability(PATH_MANAGER_CAPABILITY, PathManager.class);
 
             Optional<Supplier<DataSource>> dataSource = Optional.empty();
-            ModelNode dataSourceModel = JOURNAL_DATASOURCE.resolveModelAttribute(context, model);
-            if (dataSourceModel.isDefined()) {
-                ServiceName dataSourceCapability = context.getCapabilityServiceName(DATA_SOURCE_CAPABILITY, dataSourceModel.asString(), DataSource.class);
-                dataSource = Optional.of(serviceBuilder.requires(dataSourceCapability));
+            String dataSourceName = JOURNAL_DATASOURCE.resolveModelAttribute(context, model).asStringOrNull();
+            if (dataSourceName != null) {
+                dataSource = Optional.of(serviceBuilder.requiresCapability(DATA_SOURCE_CAPABILITY, DataSource.class, dataSourceName));
             }
             Optional<Supplier<MBeanServer>> mbeanServer = Optional.empty();
             if (context.hasOptionalCapability(JMX_CAPABILITY, ACTIVEMQ_SERVER_CAPABILITY.getDynamicName(serverName), null)) {
@@ -331,14 +315,16 @@ class ServerAdd extends AbstractAddStepHandler {
             // Inject a reference to the Elytron security domain if one has been defined.
             Optional<Supplier<SecurityDomain>> elytronSecurityDomain = Optional.empty();
             if (configuration.isSecurityEnabled()) {
-
-            final ModelNode elytronSecurityDomainModel = ELYTRON_DOMAIN.resolveModelAttribute(context, model);
-            if (elytronSecurityDomainModel.isDefined()) {
-                ServiceName elytronDomainCapability = context.getCapabilityServiceName(ELYTRON_DOMAIN_CAPABILITY, elytronSecurityDomainModel.asString(), SecurityDomain.class);
-                elytronSecurityDomain = Optional.of(serviceBuilder.requires(elytronDomainCapability));
-            } else {
-                // legacy security
-                throw ROOT_LOGGER.legacySecurityUnsupported();
+                final String elytronSecurityDomainName = ELYTRON_DOMAIN.resolveModelAttribute(context, model).asStringOrNull();
+                if (elytronSecurityDomainName != null) {
+                    elytronSecurityDomain = Optional.of(serviceBuilder.requiresCapability(ELYTRON_DOMAIN_CAPABILITY, SecurityDomain.class, elytronSecurityDomainName));
+                } else {
+                    final String legacySecurityDomain = SECURITY_DOMAIN.resolveModelAttribute(context, model).asStringOrNull();
+                    if (legacySecurityDomain == null) {
+                        throw ROOT_LOGGER.securityEnabledWithoutDomain();
+                    }
+                    // legacy security
+                    throw ROOT_LOGGER.legacySecurityUnsupported();
                 }
             }
 
@@ -346,30 +332,37 @@ class ServerAdd extends AbstractAddStepHandler {
             List<Interceptor> outgoingInterceptors = processInterceptors(OUTGOING_INTERCEPTORS.resolveModelAttribute(context, operation));
 
             // Process acceptors and connectors
-            final Set<String> socketBindingNames = new HashSet<String>();
-            TransportConfigOperationHandlers.processAcceptors(context, configuration, model, socketBindingNames);
+            final Set<String> socketBindingNames = new HashSet<>();
+            final Map<String, String> sslContextNames = new HashMap<>();
+            TransportConfigOperationHandlers.processAcceptors(context, configuration, model, socketBindingNames, sslContextNames);
 
             Map<String, Supplier<SocketBinding>> socketBindings = new HashMap<>();
             for (final String socketBindingName : socketBindingNames) {
-                Supplier<SocketBinding> socketBinding = serviceBuilder.requires(SocketBinding.JBOSS_BINDING_NAME.append(socketBindingName));
+                Supplier<SocketBinding> socketBinding = serviceBuilder.requiresCapability(SOCKET_BINDING_CAPABILITY_NAME, SocketBinding.class, socketBindingName);
                 socketBindings.put(socketBindingName, socketBinding);
             }
 
-            final Set<String> connectorsSocketBindings = new HashSet<String>();
-            configuration.setConnectorConfigurations(TransportConfigOperationHandlers.processConnectors(context, configuration.getName(), model, connectorsSocketBindings));
+            final Set<String> connectorsSocketBindings = new HashSet<>();
+            configuration.setConnectorConfigurations(TransportConfigOperationHandlers.processConnectors(context, configuration.getName(), model, connectorsSocketBindings, sslContextNames));
+
+            Map<String, Supplier<SSLContext>> sslContexts = new HashMap<>();
+            for (final Map.Entry<String, String> entry : sslContextNames.entrySet()) {
+                Supplier<SSLContext> sslContext = serviceBuilder.requiresCapability(ELYTRON_SSL_CONTEXT_CAPABILITY_NAME, SSLContext.class, entry.getValue());
+                sslContexts.put(entry.getValue(), sslContext);
+            }
 
             Map<String, Supplier<OutboundSocketBinding>> outboundSocketBindings = new HashMap<>();
             Map<String, Boolean> outbounds = TransportConfigOperationHandlers.listOutBoundSocketBinding(context, connectorsSocketBindings);
             for (final String connectorSocketBinding : connectorsSocketBindings) {
                 // find whether the connectorSocketBinding references a SocketBinding or an OutboundSocketBinding
                 if (outbounds.get(connectorSocketBinding)) {
-                    final ServiceName outboundSocketName = OutboundSocketBinding.OUTBOUND_SOCKET_BINDING_BASE_SERVICE_NAME.append(connectorSocketBinding);
-                    Supplier<OutboundSocketBinding> outboundSocketBinding = serviceBuilder.requires(outboundSocketName);
+                    final ServiceName outboundSocketName = OUTBOUND_SOCKET_BINDING_CAPABILITY.getCapabilityServiceName(connectorSocketBinding);
+                    Supplier<OutboundSocketBinding> outboundSocketBinding = serviceBuilder.requiresCapability(OUTBOUND_SOCKET_BINDING_CAPABILITY_NAME, OutboundSocketBinding.class, connectorSocketBinding);
                     outboundSocketBindings.put(connectorSocketBinding, outboundSocketBinding);
                 } else {
                     // check if the socket binding has not already been added by the acceptors
                     if (!socketBindings.containsKey(connectorSocketBinding)) {
-                        Supplier<SocketBinding> socketBinding = serviceBuilder.requires(SocketBinding.JBOSS_BINDING_NAME.append(connectorSocketBinding));
+                        Supplier<SocketBinding> socketBinding = serviceBuilder.requiresCapability(SOCKET_BINDING_CAPABILITY_NAME, SocketBinding.class, connectorSocketBinding);
                         socketBindings.put(connectorSocketBinding, socketBinding);
                     }
                 }
@@ -456,7 +449,8 @@ class ServerAdd extends AbstractAddStepHandler {
                     clusterNames,
                     elytronSecurityDomain,
                     mbeanServer,
-                    dataSource
+                    dataSource,
+                    sslContexts
             );
 
             // inject credential-references for bridges
@@ -474,7 +468,7 @@ class ServerAdd extends AbstractAddStepHandler {
                         final ServiceName queueServiceName = activeMQServiceName.append(queueConfiguration.getName());
                         final ServiceBuilder sb = context.getServiceTarget().addService(queueServiceName);
                         sb.requires(ActiveMQActivationService.getServiceName(activeMQServiceName));
-                        Supplier<ActiveMQServer> serverSupplier = sb.requires(activeMQServiceName);
+                        Supplier<ActiveMQBroker> serverSupplier = sb.requires(activeMQServiceName);
                         final QueueService queueService = new QueueService(serverSupplier, queueConfiguration, false, false);
                         sb.setInitialMode(Mode.PASSIVE);
                         sb.setInstance(queueService);
@@ -484,10 +478,9 @@ class ServerAdd extends AbstractAddStepHandler {
             }
             // Provide our custom Resource impl a ref to the ActiveMQ server so it can create child runtime resources
             ((ActiveMQServerResource) resource).setActiveMQServerServiceController(activeMQServerServiceController);
-
             // Install the JMSService
             boolean overrideInVMSecurity = OVERRIDE_IN_VM_SECURITY.resolveModelAttribute(context, operation).asBoolean();
-            JMSService.addService(serviceTarget, activeMQServiceName, overrideInVMSecurity);
+            JMSService.addService(capabilityServiceTarget, activeMQServiceName, overrideInVMSecurity);
 
             context.completeStep(OperationContext.RollbackHandler.NOOP_ROLLBACK_HANDLER);
         }
@@ -575,7 +568,7 @@ class ServerAdd extends AbstractAddStepHandler {
             configuration.setName(serverName);
             //To avoid the automatic reloading of the logging.properties by the broker.
             configuration.setConfigurationFileRefreshPeriod(-1);
-
+            configuration.setAddressQueueScanPeriod(ADDRESS_QUEUE_SCAN_PERIOD.resolveModelAttribute(context, model).asLong());
             configuration.setEnabledAsyncConnectionExecution(ASYNC_CONNECTION_EXECUTION_ENABLED.resolveModelAttribute(context, model).asBoolean());
 
             configuration.setClusterPassword(CLUSTER_PASSWORD.resolveModelAttribute(context, model).asString());
@@ -692,15 +685,24 @@ class ServerAdd extends AbstractAddStepHandler {
          *
          * @param configuration the ActiveMQ configuration
          * @param params the detyped operation parameters
-         * @throws org.jboss.as.controller.OperationFailedException
+         * @throws OperationFailedException
          */
         private void processAddressSettings(final OperationContext context, final Configuration configuration, final ModelNode params) throws OperationFailedException {
+            boolean merged = false;
             if (params.hasDefined(ADDRESS_SETTING)) {
                 for (final Property property : params.get(ADDRESS_SETTING).asPropertyList()) {
                     final String match = property.getName();
                     final ModelNode config = property.getValue();
-                    final AddressSettings settings = AddressSettingAdd.createSettings(context, config);
-                    configuration.getAddressesSettings().put(match, settings);
+                    boolean isRootAddressMatch = configuration.getWildcardConfiguration().getAnyWordsString().equals(match);
+                    final AddressSettings settings = AddressSettingAdd.createSettings(context, config, isRootAddressMatch);
+                    if (!merged && isRootAddressMatch) {
+                        settings.merge(createDefaulAddressSettings());
+                        merged = true;
+                    }
+                    configuration.addAddressSetting(match, settings);
+                }
+                if (!merged) {
+                    configuration.addAddressSetting(configuration.getWildcardConfiguration().getAnyWordsString(), createDefaulAddressSettings());
                 }
             }
         }
